;;; $DOOMDIR/config.el -*- lexical-binding: t; -*-

;; Some functionality uses this to identify you
(setq user-full-name "FollieHiyuki"
      user-mail-address "folliekazetani@protonmail.com")

;; Font settings
(setq doom-font (font-spec :family "Iosevka Nerd Font" :size 16)
      doom-variable-pitch-font (font-spec :family "sans" :size 16)
      doom-big-font (font-spec :family "Iosevka Nerd Font" :size 24)
      doom-unicode-font (font-spec :family "Symbola" :size 14))
(custom-set-faces!
  '(font-lock-comment-face :slant italic))

;; Might speedup all-the-icons
;;(setq inhibit-compacting-font-caches t)

;; Emoji
(after! emojify
  (setq emojify-display-style 'unicode))

;; Theme
(setq doom-theme 'doom-nord)
(setq doom-themes-enable-bold t
      doom-themes-enable-italic t)
(after! treemacs
  (setq doom-themes-treemacs-theme 'doom-colors
        doom-themes-treemacs-enable-variable-pitch nil))

;; Pick an image for dashboard
;;(defun random-choice (items)
;;      (let* ((size (length items))
;;            (index (random size)))
;;      (nth index items)))
;;(defvar *kawaii* (random-choice (delete "."
;;      (delete ".." (directory-files (expand-file-name "images" doom-private-dir))))))
(setq +doom-dashboard-banner-file "yukino.png"
      +doom-dashboard-banner-dir (expand-file-name "images" doom-private-dir)
      +doom-dashboard-banner-padding '(0 . 2)
      +doom-dashboard-functions '(doom-dashboard-widget-banner
                                  doom-dashboard-widget-shortmenu
                                  doom-dashboard-widget-loaded))

;; Modeline
(setq doom-modeline-unicode-fallback t)

;; Line number
(setq display-line-numbers-type 'relative)

;; Indent
(setq-default
      indent-tabs-mode nil
      tab-width 2
      evil-shift-width 2
      standard-indent 2
      line-spacing 2)
(after! highlight-indent-guides
  (setq highlight-indent-guides-method 'character
        highlight-indent-guides-responsive t))

;; Use ranger.el instead of default dired
(after! ranger
  (setq ranger-override-dired-mode t
        ranger-parent-depth 0
        ranger-cleanup-eagerly t
        ranger-show-hidden t
        ranger-max-preview-size 20
        ranger-dont-show-binary t))

;; Projectile
(after! projectile
  (setq projectile-project-search-path '("~/Code/")
        projectile-auto-discover nil))

;; Counsel
(after! counsel
  (setq counsel-linux-app-format-function #'counsel-linux-app-format-function-name-only))
(map! "C-M-j" 'counsel-switch-buffer)
(map! :map minibuffer-local-map
      "C-r" 'counsel-minibuffer-history)

;; Completion + lsp
(after! company
  (setq company-idle-delay 0.5
        company-minimum-prefix-length 2)
  (setq company-show-numbers nil))
(setq lsp-clients-clangd-args '("-j=2"
                                "--background-index"
                                "--clang-tidy"
                                "--completion-style=detailed"
                                "--pch-storage=memory"
                                "--header-insertion=iwyu"
                                "--header-insertion-decorators"))
;;(after! lsp-clangd (set-lsp-priority! 'clangd 2))  ;; Prefer clangd instead of default ccls

;; Quicker which-key
(after! which-key
  (setq which-key-idle-delay 0.5))

;; Org mode
(setq org-directory "~/Documents/Org/"
      org-agenda-files '("~/Documents/Org/agenda.org")
      org-default-notes-file (expand-file-name "notes.org" org-directory)
      org-journal-dir "~/Documents/Org/Journal/"
      org-hide-emphasis-markers t)

;; Spell checker
(after! flyspell
  (setq flyspell-lazy-idle-seconds 2))

;; Elfeed
(setq rmh-elfeed-org-files (list (expand-file-name "elfeed.org" doom-private-dir)))
;;(add-hook! 'elfeed-search-mode-hook 'elfeed-update)

;; Asynchronous bytecode compilation
(use-package! async
  :defer t
  :init
  (dired-async-mode 1)
  (async-bytecomp-package-mode 1)
  :custom (async-bytecomp-allowed-packages '(all)))

;; Set qutebrowser as the default browser
(setq browse-url-browser-function 'browse-url-generic
      browse-url-generic-program "qutebrowser")

;; Edit buffer as sudo
;;(defun follie/sudo-edit (&optional arg)
;;  "Edit file as super-user with optional ARG."
;;  (interactive "P")
;;  (if (or arg (not buffer-file-name))
;;      (find-file (concat "/sudo::" (read-file-name "File (as root): ")))
;;    (find-alternate-file (concat "/sudo::" buffer-file-name))))

;; Here are some additional functions/macros that could help you configure Doom:
;;
;; - `load!' for loading external *.el files relative to this one
;; - `use-package!' for configuring packages
;; - `after!' for running code after a package has loaded
;; - `add-load-path!' for adding directories to the `load-path', relative to
;;   this file. Emacs searches the `load-path' when you load packages with
;;   `require' or `use-package'.
;; - `map!' for binding new keys
;;
;; To get information about any of these functions/macros, move the cursor over
;; the highlighted symbol at press 'K' (non-evil users must press 'C-c c k').
;; This will open documentation for it, including demos of how they are used.
;;
;; You can also try 'gd' (or 'C-c c d') to jump to their definition and see how
;; they are implemented.
